'''

This crib sheet shows how to load MMS MEC data

'''

from pyspedas import mms_load_mec
from pytplot import tplot, del_data

# 'srvy' mode data for MMS1 are loaded by default
mms_load_mec(trange=['2015-10-16', '2015-10-17'])

tplot(['mms1_mec_r_gsm', 'mms1_mec_v_gsm'])




from pytplot import options
from pyspedas import tnames

def mms_hpca_set_metadata(probe='1', fov=[0, 360], suffix=''):
    prefix = 'mms'+str(probe)
    valid_density = prefix+'*_number_density'+suffix
    valid_vel = prefix+'*_ion_bulk_velocity*'+suffix
    valid_temp = prefix+'*_scalar_temperature'+suffix

    for var in tnames(valid_density):
        if var == prefix+'_hpca_hplus_number_density'+suffix: options(var, 'ytitle', 'H+ density')
        if var == prefix+'_hpca_heplus_number_density'+suffix: options(var, 'ytitle', 'He+ density')
        if var == prefix+'_hpca_heplusplus_number_density'+suffix: options(var, 'ytitle', 'He++ density')
        if var == prefix+'_hpca_oplus_number_density'+suffix: options(var, 'ytitle', 'O+ density')
        if var == prefix+'_hpca_oplusplus_number_density'+suffix: options(var, 'ytitle', 'O++ density')

    for var in tnames(valid_vel):
        if var == prefix+'_hpca_hplus_ion_bulk_velocity'+suffix:
            options(var, 'legend_names', ['Vx (H+)', 'Vy (H+)', 'Vz (H+)'])
            options(var, 'color', ['b', 'g', 'r'])
            options(var, 'ytitle', 'H+ velocity')
        if var == prefix+'_hpca_heplus_ion_bulk_velocity'+suffix:
            options(var, 'legend_names', ['Vx (He+)', 'Vy (He+)', 'Vz (He+)'])
            options(var, 'color', ['b', 'g', 'r'])
            options(var, 'ytitle', 'He+ velocity')
        if var == prefix+'_hpca_heplusplus_ion_bulk_velocity'+suffix:
            options(var, 'legend_names', ['Vx (He++)', 'Vy (He++)', 'Vz (He++)'])
            options(var, 'color', ['b', 'g', 'r'])
            options(var, 'ytitle', 'He++ velocity')
        if var == prefix+'_hpca_oplus_ion_bulk_velocity'+suffix:
            options(var, 'legend_names', ['Vx (O+)', 'Vy (O+)', 'Vz (O+)'])
            options(var, 'color', ['b', 'g', 'r'])
            options(var, 'ytitle', 'O+ velocity')
        if var == prefix+'_hpca_oplusplus_ion_bulk_velocity'+suffix:
            options(var, 'legend_names', ['Vx (O++)', 'Vy (O++)', 'Vz (O++)'])
            options(var, 'color', ['b', 'g', 'r'])
            options(var, 'ytitle', 'O++ velocity')

    for var in tnames(valid_temp):
        if var == prefix+'_hpca_hplus_scalar_temperature'+suffix: options(var, 'ytitle', 'H+ temp (eV)')
        if var == prefix+'_hpca_heplus_scalar_temperature'+suffix: options(var, 'ytitle', 'He+ temp (eV)')
        if var == prefix+'_hpca_heplusplus_scalar_temperature'+suffix: options(var, 'ytitle', 'He++ temp (eV)')
        if var == prefix+'_hpca_oplus_scalar_temperature'+suffix: options(var, 'ytitle', 'O+ temp (eV)')
        if var == prefix+'_hpca_oplusplus_scalar_temperature'+suffix: options(var, 'ytitle', 'O++ temp (eV)')





'''
 PURPOSE:
       This function modifies the energy table in FEEPS spectra (intensity, count_rate, counts) variables
       using the function: mms_feeps_energy_table (which is s/c, sensor head and sensor ID dependent)

 NOTES:
     BAD EYES are replaced by NaNs

'''

import pytplot
from pytplot import get_data, store_data
from .mms_feeps_energy_table import mms_feeps_energy_table
from pyspedas import tnames

def mms_feeps_correct_energies(probes, data_rate, level='l2', suffix=''):

    types = ['top', 'bottom']
    sensors = range(1, 13)
    units_types = ['intensity', 'count_rate']

    for probe in probes:
        for sensor_type in types:
            for sensor in sensors:
                if sensor >= 6 and sensor <= 8: 
                    species = 'ion'
                else: 
                    species = 'electron'

                for units in units_types:
                    var_name = tnames('mms'+probe+'_epd_feeps_'+data_rate+'_'+level+'_'+species+'_'+sensor_type+'_'+units+'_sensorid_'+str(sensor))

                    if var_name == []:
                        continue
                    else:
                        var_name = var_name[0]

                    var_data = get_data(var_name+suffix)
                    if var_data is not None:
                        times, data = var_data
                    else:
                        continue

                    energies = pytplot.data_quants[var_name+suffix].spec_bins.values
                    energy_map = mms_feeps_energy_table(probe, sensor_type[0:3], sensor)

                    try:
                        store_data(var_name+suffix, data={'x': times, 'y': data, 'v': energy_map})
                    except:
                        continue


# -*- coding: utf-8 -*-
"""
File:
    tinterpol.py

Desrciption:
    Interpolates data.

Parameters:
    names: str/list of str
        List of pytplot names.
    interp_names: str/list of str
        List of pytplot names. This should contain the new times array.
    method: str
        Interpolation method. Default is ‘linear’.
        Specifies the kind of interpolation as a string (‘linear’, ‘nearest’,
        ‘zero’, ‘slinear’, ‘quadratic’, ‘cubic’, ‘previous’, ‘next’)
        where ‘zero’, ‘slinear’, ‘quadratic’ and ‘cubic’ refer to a spline
        interpolation of zeroth, first, second or third order; ‘previous’ and
        ‘next’ simply return the previous or next value of the point) or
        as an integer specifying the order of the spline interpolator to use.
    new_names: str/list of str
        List of new_names for pytplot variables.
        If '', then pytplot variables are replaced.
        If not given, then a suffix is applied.
    suffix:
        A suffix to apply. Default is '-itrp'.

Notes:
    Allowed wildcards are ? for a single character, * from multiple characters.
    Similar to tinterpol in IDL SPEDAS.
"""

import pyspedas
import pytplot
import numpy
from scipy.interpolate import interp1d


def tinterpol(names, interp_names=None, method=None, new_names=None,
              suffix=None):

    old_names = pyspedas.tnames(names)

    if len(old_names) < 1:
        print('tinterpol error: No pytplot names were provided.')
        return

    if suffix is None:
        suffix = '-itrp'

    if method is None:
        method = 'linear'

    if new_names is None:
        n_names = [s + suffix for s in old_names]
    elif new_names == '':
        n_names = old_names
    else:
        n_names = new_names

    if len(n_names) != len(old_names):
        n_names = [s + suffix for s in old_names]

    for i in range(len(old_names)):
        alldata = pytplot.get_data(old_names[i])
        time = alldata[0]
        data = alldata[1]
        alldata1 = pytplot.get_data(interp_names[i])        
        new_time = alldata1[0]
        new_data = alldata1[1]
        data = numpy.asarray(data).squeeze()
        f2 = interp1d(time, data, kind=method)
        new_data = f2(new_time)
        pytplot.store_data(n_names[i], data={'x': new_time, 'y': new_data})
        print('tinterpol (' + method + ') was applied to: ' + n_names[i])




import re
from dateutil.parser import parse
from bisect import bisect_left

def mms_files_in_interval(in_files, trange):

    file_name = 'mms.*_([0-9]{8,14})_v(\d+).(\d+).(\d+).cdf'

    file_times = []

    regex = re.compile(file_name)

    for file in in_files:
        matches = regex.match(file['file_name'])
        if matches:
            file_times.append((file['file_name'], parse(matches.groups()[0]).timestamp(), file['timetag'], file['file_size']))

    # sort in time
    sorted_files = sorted(file_times, key=lambda x: x[1])

    times = [t[1] for t in sorted_files]

    idx_min = bisect_left(times, parse(trange[0]).timestamp())

    # note: purposefully liberal here; include one extra file so that we always get the burst mode data
    if idx_min == 0:
        return [{'file_name': f[0], 'timetag': f[2], 'file_size': f[3]} for f in sorted_files[idx_min:]]
    else:
        return [{'file_name': f[0], 'timetag': f[2], 'file_size': f[3]} for f in sorted_files[idx_min-1:]]



'''
  This crib sheet produces the MMS basic dayside figure

'''

from pyspedas import mms_load_fgm, mms_load_mec, mms_load_edp, mms_load_dsp, mms_load_fpi, mms_load_hpca
from pytplot import tplot, options, get_data, store_data
import numpy as np

probe = '1'
trange = ['2015-10-16', '2015-10-17']

mms_load_fgm(probe=probe, data_rate='srvy', trange=trange)
mms_load_mec(probe=probe, data_rate='srvy', trange=trange)
mms_load_fpi(probe=probe, data_rate='fast', datatype=['des-moms', 'dis-moms'], trange=trange)
mms_load_edp(probe=probe, datatype='scpot', trange=trange)
mms_load_edp(probe=probe, data_rate='fast', datatype='dce', trange=trange)
mms_load_edp(probe=probe, data_rate='srvy', datatype=['dce', 'hfesp'], trange=trange)
mms_load_dsp(probe=probe, data_rate='fast', datatype='bpsd', trange=trange)
mms_load_hpca(probe=probe, data_rate='srvy', datatype='ion', trange=trange)

times, data = get_data('mms'+probe+'_fgm_b_gsm_srvy_l2')

store_data('mms'+probe+'_b_gsm_vec', data={'x': times, 'y': data[:,0:3]}) # B-field vector
store_data('mms'+probe+'_b_gsm_mag', data={'x': times, 'y': data[:,3]}) # B-field magnitude

times, data = get_data('mms'+probe+'_edp_scpot_fast_l2')
store_data('mms'+probe+'_edp_fast_scpot_ln', data={'x': times, 'y': np.log(data)})

options('mms'+probe+'_dsp_bpsd_omni_fast_l2', 'ylog', True)
options('mms'+probe+'_dsp_bpsd_omni_fast_l2', 'zlog', True)
options('mms'+probe+'_dsp_bpsd_omni_fast_l2', 'Colormap', 'jet')
options('mms'+probe+'_edp_hfesp_srvy_l2', 'ylog', True)
options('mms'+probe+'_edp_hfesp_srvy_l2', 'zlog', True)
options('mms'+probe+'_edp_hfesp_srvy_l2', 'Colormap', 'jet')

tplot(['mms'+probe+'_b_gsm_vec', 
       'mms'+probe+'_b_gsm_mag', 
       'mms'+probe+'_dis_energyspectr_omni_fast', 
       'mms'+probe+'_des_energyspectr_omni_fast', 
       'mms'+probe+'_dis_numberdensity_fast', 
       'mms'+probe+'_edp_fast_scpot_ln', 
       'mms'+probe+'_dis_bulkv_gse_fast', 
                    # '_exb_vperp_z', 
                    # '_hpca_hplus_flux_elev_0-360', 
                    # '_hpca_oplus_flux_elev_0-360', 
        'mms'+probe+'_edp_dce_gse_fast_l2', 
        'mms'+probe+'_edp_hfesp_srvy_l2', 
        'mms'+probe+'_dsp_bpsd_omni_fast_l2'])



from pyspedas import tnames
from pytplot import options, get_data, store_data

def mms_hpca_elevations():
    anode_theta = [123.75000, 101.25000, 78.750000, 56.250000, 33.750000, 
        11.250000, 11.250000, 33.750000, 56.250000, 78.750000, 
        101.25000, 123.75000, 146.25000, 168.75000, 168.75000, 
        146.25000]
    anode_theta[6:14] = [anode_val+180. for anode_val in anode_theta[6:14]]
    return anode_theta

def mms_hpca_anodes(fov=[0, 360]):
    anodes = mms_hpca_elevations()
    return [i for i, anode in enumerate(anodes) if anode >= float(fov[0]) and anode <= float(fov[1])]


def mms_hpca_sum_fov(times, data, angles, energies, fov=[0, 360], anodes=None):
    anodes_in_fov = mms_hpca_anodes(fov=fov)
    data_within_fov = data[:,anodes_in_fov,:]
    return data_within_fov.sum(axis=1)

def mms_hpca_avg_fov(times, data, angles, energies, fov=[0, 360], anodes=None):
    anodes_in_fov = mms_hpca_anodes(fov=fov)
    data_within_fov = data[:,anodes_in_fov,:]
    return data_within_fov.mean(axis=1)

def mms_hpca_calc_anodes(fov=[0, 360], probe='1', suffix=''):
    """
    This function will sum (or average, for flux) the HPCA data over the requested field-of-view (fov)
    Parameters:
        fov : list of int
            field of view, in angles, from 0-360
        probe : str
            probe #, e.g., '4' for MMS4
        suffix: str
            suffix of the loaded data

    Returns:
        List of tplot variables created.
    """
    sum_anodes = [a+suffix for a in ['*_count_rate', '*_RF_corrected', '*_bkgd_corrected', '*_norm_counts']]
    avg_anodes = ['*_flux'+suffix]
    output_vars = []

    fov_str = '_elev_'+str(fov[0])+'-'+str(fov[1])

    for sum_anode in sum_anodes:
        vars_to_sum = tnames(sum_anode)

        for var in vars_to_sum:
            times, data, angles, energies = get_data(var)

            updated_spectra = mms_hpca_sum_fov(times, data, angles, energies, fov=fov)

            store_data(var+fov_str, data={'x': times, 'y': updated_spectra, 'v': energies})
            options(var+fov_str, 'spec', True)
            options(var+fov_str, 'ylog', True)
            options(var+fov_str, 'zlog', True)
            options(var+fov_str, 'Colormap', 'jet')
            output_vars.append(var+fov_str)

    for avg_anode in avg_anodes:
        vars_to_avg = tnames(avg_anode)

        for var in vars_to_avg:
            times, data, angles, energies = get_data(var)

            updated_spectra = mms_hpca_avg_fov(times, data, angles, energies, fov=fov)

            store_data(var+fov_str, data={'x': times, 'y': updated_spectra, 'v': energies})
            options(var+fov_str, 'spec', True)
            options(var+fov_str, 'ylog', True)
            options(var+fov_str, 'zlog', True)
            options(var+fov_str, 'Colormap', 'jet')
            output_vars.append(var+fov_str)




# -*- coding: utf-8 -*-
"""
File:
    ex_spectra.py

Desrciption:
    Basic example of plotting a spectrogram with THEMIS data.
    Downloads THEMIS data and plots a spectrogram.

"""

import pyspedas
import pytplot


def ex_spectra():
    # Delete any existing pytplot variables
    pytplot.del_data()
    # Download THEMIS data for 2015-12-31
    pyspedas.load_data('themis', ['2015-12-31'], ['tha'], 'sst', 'l2')
    # Specify options
    pytplot.tplot_options('title', 'tha_psif_en_eflux 2015-12-31')
    pytplot.ylim('tha_psif_en_eflux', 10000.0, 10000000.0)
    pytplot.options('tha_psif_en_eflux', 'colormap', 'viridis')
    # Plot spectrogram
    pytplot.tplot(['tha_psif_en_eflux'])

# Run the example code
# ex_spectra()
